/*
 *  Copyright (C) 2001-2006, INRIA, Universite de Rennes 1, EDF.
 */

#include <stdlib.h>
#include <stdio.h>
#include <sys/types.h>
#include <signal.h>
#include <errno.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <string.h>

#include <getopt.h>
#include <time.h>
#include <kerrighed.h>

#define CHKPT_DIR "/var/chkpt/"

short from_appid = 0;
media_t media = DISK;
int sig = 0;
char * description = NULL;


void show_help(void)
{
	printf("Usage : checkpoint [-h] "
	       "[-k | --kill | -kSIGNAL | --kill=SIGNAL] "
	       "[a|--from-appid] "
	       "[(-m|--media) disk|memory] "
	       "[(-d|--description) description] "
	       "pid \n");
}

void parse_args(int argc, char *argv[])
{
	char c;
	int option_index = 0;
	char * short_options= "ham:d:k::";
	static struct option long_options[] = {
		{"help", no_argument, 0, 'h'},
		{"from-appid", no_argument, 0, 'a'},
		{"media", required_argument, 0, 'm'},
		{"description", required_argument, 0, 'd'},
		{"kill", optional_argument, 0, 'k'},
		{0, 0, 0, 0}
	};

	while ((c = getopt_long(argc, argv, short_options,
				long_options, &option_index)) != -1) {
		switch (c) {
		case 'h':
			show_help();
			return;
		case 'a':
			from_appid=1;
			break;
		case 'm':
			if (strcmp(optarg, "DISK") == 0)
				media = DISK;
			else if (strcmp(optarg, "MEMORY") == 0)
				media = MEMORY;
			else printf("Warning: Unknown type of media\n");
			break;
		case 'd':
			description = optarg;
			break;
		case 'k':
			if (optarg)
				sig = atoi(optarg);
			else 
				sig = SIGTERM;
			break;
		default:
			printf("Warning: Option %c "
			       "not implemented\n", c);
			break;
		}
	}
}


void check_environment(void)
{
	struct stat buffer;
	int status;

	/* is Kerrighed launched ? */
	if (get_nr_cpu() == -1)
	{
		fprintf(stderr, "no kerrighed nodes found\n");
		exit(-EPERM);
	}	

	/* /var/chkpt exists ? */
	status = stat(CHKPT_DIR, &buffer);
	if (status) {
		perror(CHKPT_DIR);
		exit(-ENOENT);
	}
}

int write_description(char * description, 
		      checkpoint_infos_t * infos)
{
	if (description == NULL) 
		description = "No description";

	time_t date = time(NULL);

	printf("Identifier: %ld\n"
	       "Version: %d\n"
	       "Description: %s\n"
	       "Date: %s\n",
	       infos->app_id,
	       infos->chkpt_sn,
	       description,
	       ctime(&date));

	// need to write it in a file
	FILE* fd;

	char path[256];
	sprintf(path, "%s/%ld/description_v%d.txt", CHKPT_DIR,
		infos->app_id, infos->chkpt_sn);

	if ((fd = fopen(path, "a")) == NULL)  {
		perror(path);
		return -1;
	} else {
		fprintf(fd,
			"Identifier: %ld\n"
			"Version: %d\n"
			"Description: %s\n"
			"Date: %ld\n",
			infos->app_id,
			infos->chkpt_sn,
			description,
			date);
		fclose(fd);
	}
	return 0;
}

int main(int argc, char *argv[])
{
	int r = 0;
	int pid = -1;

	/* Check environment */
	check_environment();

	/* Manage options with getopt */
	if (argc == 1) {
		show_help();
		return 0;
	}
	parse_args(argc, argv);

	/* get the pid */
	pid = atoi( argv[argc-1] );
	if (pid < 2) {
		r = -EINVAL;
		goto exit;
	}

	checkpoint_infos_t infos;
	if (from_appid) {
		printf("Checkpointing application %d...\n", pid);
		infos = application_checkpoint_from_appid(media, pid, sig);
	} else {
		printf("Checkpointing application in which "
		       "process %d is involved...\n", pid);
		infos = application_checkpoint_from_pid(media, pid, sig);
	}

	r = infos.result;

	if (!r)
		write_description(description, &infos);

exit:
	if (r!=0)
		r = errno;

	switch (r) {
	case 0:
		break;
	case E_CR_TASKDEAD:
		fprintf(stderr, "checkpoint: can not checkpoint an application with dead/zombie process\n");
		break;
	default:
		perror("checkpoint");
		break;
	}

	return r;
}
